/* ***** BEGIN LICENSE BLOCK *****
 *   Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is ThumbsDown.
 *
 * The Initial Developer of the Original Code is
 * Atte Kemppilä.
 * Portions created by the Initial Developer are Copyright (C) 2008
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 * 
 * ***** END LICENSE BLOCK ***** */


const CLASS_ID = Components.ID("594fe6be-0f6b-4aae-a19d-570543633cb2");
const CLASS_NAME = "ThumbsDown Script Manager";
const CONTRACT_ID = "@thumbsdown.mozdev.org/script-manager;1";
const CATEGORY_ENTRY = "thumbsdown";

const Cc = Components.classes;
const Ci = Components.interfaces;
const Cr = Components.results;

const TYPE_DIRECT = 1;
const TYPE_REGEXP = 2;
const TYPE_CUSTOM = 3;
const TYPE_WRAPPED = 4;

const TYPE_REGEXP_THUMBNAIL = 1;
const TYPE_REGEXP_LINK      = 2;

const TYPE_WRAPPED_ID     = 1;
const TYPE_WRAPPED_REGEXP = 2;


function ScriptManager()
{
    this.wrappedJSObject = this;
    this._scripts = null;
    this.prevMatch = null;
}

ScriptManager.prototype =
{
    get sqlExec()
    {
        if (! this._sqlExec)
        {
            var loader = Cc["@mozilla.org/moz/jssubscript-loader;1"].getService(Ci.mozIJSSubScriptLoader);
            var sqlConfig = {};
            loader.loadSubScript("chrome://thumbsdown/content/sql/SqlExecutor.js");
            loader.loadSubScript("chrome://thumbsdown/content/sql/sqlConfig.js", sqlConfig);
            sqlConfig = sqlConfig.sqlConfig;
            
            this._sqlExec = new SqlExecutor(sqlConfig.databaseFilename, sqlConfig.statements, sqlConfig.schemaVersion);
        }
        return this._sqlExec;
    },
    
    get scripts()
    {
        if (this._scripts == null)
            this._scripts = this._loadScripts();
        return this._scripts;
    },
    
    reset: function()
    {
        this._scripts = null;
        this.prevMatch = null;
    },
    
    addScript: function(script)
    {
        var id = this.sqlExec.execute("insertScript", script);
        this.reset();
        return id;
    },
    
    getScript: function(id)
    {
        var result = this.sqlExec.fetchOne("getScript", id);
        var script = new Script(result);
        return script;
    },
    
    setScript: function(script)
    {
        this.sqlExec.execute("updateScript", script);
        this.reset();
    },
    
    removeScript: function(id)
    {
        this.sqlExec.execute("removeScript", id);
        this.reset();
    },
    
    lookup: function(linkUrl, thumbnailUrl)
    {
        var script = this._lookup(linkUrl, thumbnailUrl, this.prevMatch);
        if (script)
            this.prevMatch = script;
        return script;
    },
    
    lookupAll: function(linkUrl, thumbnailUrl)
    {
        var matches = [];
        for each (var script in this.scripts)
            if (this._match(script, linkUrl, thumbnailUrl))
                matches.push(script);
        return matches;
    },
    
    _lookup: function(linkUrl, thumbnailUrl, script)
    {
        if (script)
            if (this._match(script, linkUrl, thumbnailUrl))
                return script;
        for each (var script in this.scripts)
            if (this._match(script, linkUrl, thumbnailUrl))
                return script;
    },
    
    _match: function(script, linkUrl, thumbnailUrl)
    {
        if (! script.re_matcher.test(linkUrl))
            return false;
        
        switch (script.type)
        {
            case TYPE_DIRECT:
            case TYPE_CUSTOM:
            case TYPE_WRAPPED:
                return true;
            
            case TYPE_REGEXP:
                switch (script.subType)
                {
                    case TYPE_REGEXP_THUMBNAIL:
                        return (thumbnailUrl && script.re_pattern.test(thumbnailUrl));
                    case TYPE_REGEXP_LINK:
                        return script.re_pattern.test(linkUrl);
                }
                break;
        }
        
        throw new Error("unreachable code");
    },
    
    _loadScripts: function()
    {
        var scripts1 = this.sqlExec.fetchAll("getScripts");
        var scripts2 = [];
        for each (var script in scripts1)
            scripts2.push(new Script(script));
        return scripts2;
    },
    
    // nsISupports
    QueryInterface: function(iid)
    {
        if (iid.equals(Ci.nsISupports) ||
            iid.equals(Ci.nsISupportsWeakReference))
            return this;
        throw Cr.NS_ERROR_NO_INTERFACE;
    }
}


function Script(script)
{
    this.id = script.id;
    this.name = script.name;
    this.matcher = script.matcher;
    this.type = script.type;
    this.subType = script.subType;
    this.action1 = script.action1;
    this.action2 = script.action2;
}

Script.prototype =
{
    get re_matcher() { if (! this._re_matcher) this._re_matcher = new RegExp("^" + this.matcher + "$"); return this._re_matcher; },
    
    // regexp
    get source()   { return this.subType; },
    get pattern()  { return this.action1; },
    get template() { return this.action2; },
    
    get re_pattern() { if (! this._re_pattern) this._re_pattern = new RegExp("^" + this.pattern + "$"); return this._re_pattern; },
    
    // custom
    get script() { return this.action1; },
    
    // wrapped
    get wrappedType() { return this.subType; },
    get wrappedExpr() { return this.action1; },
    
    get re_wrapped() { if (! this._re_wrapped) this._re_wrapped = new RegExp("^" + this.wrappedExpr + "$"); return this._re_wrapped; }
}


// nsIFactory
var ScriptManagerFactory =
{
    instance: null,
    
    createInstance: function(outer, iid)
    {
        if (outer != null)
            throw Cr.NS_ERROR_NO_AGGREGATION;
        if (this.instance == null)
            this.instance = new ScriptManager();
        return this.instance.QueryInterface(iid);
    }
};

// nsIModule
var ScriptManagerModule =
{
    registerSelf: function(compMgr, fileSpec, location, type)
    {
        // register
        compMgr = compMgr.QueryInterface(Ci.nsIComponentRegistrar);
        compMgr.registerFactoryLocation(CLASS_ID, CLASS_NAME, CONTRACT_ID, fileSpec, location, type);
        // add start-up observer
        //var catMgr = Cc["@mozilla.org/categorymanager;1"].getService(Ci.nsICategoryManager);
        //catMgr.addCategoryEntry("xpcom-startup", CATEGORY_ENTRY, CONTRACT_ID, true, true);
    },
    
    unregisterSelf: function(compMgr, fileSpec, location)
    {
        // remove start-up observer
        //var catMgr = Cc["@mozilla.org/categorymanager;1"].getService(Ci.nsICategoryManager);
        //catMgr.deleteCategoryEntry("xpcom-startup", CATEGORY_ENTRY, true);
        // unregister
        compMgr = compMgr.QueryInterface(Ci.nsIComponentRegistrar);
        compMgr.unregisterFactoryLocation(CLASS_ID, location);
    },
    
    getClassObject: function(compMgr, cid, iid)
    {
        if (! cid.equals(CLASS_ID))
            throw Cr.NS_ERROR_NO_INTERFACE;
        if (! iid.equals(Ci.nsIFactory))
            throw Cr.NS_ERROR_NOT_IMPLEMENTED;
        return ScriptManagerFactory;
    },
    
    canUnload: function(compMgr)
    {
        return true;
    }
};

function NSGetModule(compMgr, fileSpec)
{
    return ScriptManagerModule;
}
